# 2008-01-03--16-41-13 HIYA



# etprof - trivial exsclusive time profiler
 # I wrote this because my feeling is that exclusiveRunTime info
 # of the TclLib's profiler is broken.
 #
 # Usage: source it as first line in your application
 #        and check the standard output for profiling information
 #        at runtime.
 #
 # Copyright (C) 2004 Salvatore Sanfilippo

 package require Tcl 8.4

 namespace eval ::etprof {}

 # Unset the specified var and returns the old content
 proc ::etprof::getunset varName {
     upvar $varName var
     set t $var
     unset var
     return $t
 }

 proc ::etprof::lpop listVar {
     upvar $listVar list
     set retval [lindex $list end]
     set list [lrange [::etprof::getunset list] 0 end-1]
     return $retval
 }

 proc ::etprof::TraceHandler {name cmd args} {
     # We need to misure the elapsed time as early as possible.
     set enter_clicks [clock clicks]
     set elapsed [expr {$enter_clicks-$::etprof::timer}]

     #####################################################################
     # Starting from this point it's possible to do potentially slow
     # operations. They will not be accounted as time spent in procedures
     #####################################################################

     # Show profiling information every 50000 calls to the profiler
     if {[incr ::etprof::hits] >= 50000} {
        set ::etprof::hits 0
        ::etprof::printLiveInfo
     }

     # The following is a flag that may be turned on inside [switch].
     # If true the [clock clicks] value will be added to the cumulative_timers
     # list as later as possible before to exit this function.
     set non_recursive_enter 0;
     switch -- [lindex $args end] {
        enter {
            # Try to guess the caller function. If we are at toplevel
            # set it to TOPLEVEL, in the hope there isn't another function
            # with this name.
            if {[info level] == 1} {
                set caller TOPLEVEL
            } else {
                # Otherwise use [info level] to get the caller name.
                set caller [lindex [info level -1] 0]
                set callerns [uplevel 1 namespace current]
                set caller [::etprof::fullyQualifiedName $caller $callerns]
            }

            set depth [incr ::etprof::depth($name)]
            ::etprof::enterHandler $name $caller $elapsed
            if {$depth == 1} {
                set non_recursive_enter 1
            }
        }
        leave {
            if {$::etprof::depth($name) == 1} {
                set t [lpop ::etprof::cumulative_timers]
                set cum_elapsed [expr {$enter_clicks-$t}]
                incr ::etprof::cumulative($name) $cum_elapsed
            }
            ::etprof::leaveHandler $name $elapsed
            incr ::etprof::depth($name) -1
        }
     }

     #####################################################################
     # Don't add slow operations after this comment.
     # The following lines should only be used to get [clock clicks]
     # values at near as possible to the leave from this function.
     #####################################################################

     # Add the time spent inside the handler to every element
     # of the cumulative timers list. Note that the time needed
     # to perform the following operation will be accounted to user code
     # as comulative, but from worst-case tests performed this does not
     # seems to alter the output in a significant way.
     if {[llength $::etprof::cumulative_timers]} {
        set spent [expr {[clock clicks]-$enter_clicks}]
        foreach t $::etprof::cumulative_timers {
            lappend newlist [expr {$t+$spent}]
        }
        set ::etprof::cumulative_timers $newlist
     }
     # Note that we take the 'timer' sample as the last operation.
     # Basically this profiler try to be more accurate in
     # the exclusive measure.
     if {$non_recursive_enter} {
                lappend ::etprof::cumulative_timers [clock clicks]
     }
     set ::etprof::timer [clock clicks]
 }

 proc ::etprof::enterHandler {name caller elapsed} {
     # The caller may not exists in the arrays because may be a built-in
     # like [eval].
     if {[info exists ::etprof::exclusive($caller)]} {
        incr ::etprof::exclusive($caller) $elapsed
     }
     incr ::etprof::calls($name)
 }

 proc ::etprof::leaveHandler {name elapsed} {
     # The program is leaving the function. Add the current time value
     # to it. And reset the value.
     incr ::etprof::exclusive($name) $elapsed
 }

 # That comes from the TclLib's profiler, seems working but
 # I wonder if there is a better (faster) way to get the fully-qualified
 # names.
 proc ::etprof::fullyQualifiedName {name ns} {
     if { ![string equal $ns "::"] } {
        if { ![string match "::*" $name] } {
            set name "${ns}::${name}"
        }
     }
     if { ![string match "::*" $name] } {
        set name "::$name"
     }
     return $name
 }

 # That comes from the TclLib's profiler, seems working but
 # I wonder if there is a better way to get the fully-qualified
 # name of the procedure to create without pattern matching.
 proc ::etprof::profProc {name arglist body} {
     # Get the fully qualified name of the proc
     set ns [uplevel [list namespace current]]
     set name [::etprof::fullyQualifiedName $name $ns]
     # If the proc call did not happen at the global context and it did not
     # have an absolute namespace qualifier, we have to prepend the current
     # namespace to the command name
     if { ![string equal $ns "::"] } {
        if { ![string match "::*" $name] } {
            set name "${ns}::${name}"
        }
     }
     if { ![string match "::*" $name] } {
        set name "::$name"
     }

     uplevel 1 [list ::etprof::oldProc $name $arglist $body]
     trace add execution $name {enter leave} \
              [list ::etprof::TraceHandler $name]
     ::etprof::initProcInfo $name
     return
 }

 proc ::etprof::initProcInfo name {
     set ::etprof::calls($name) 0
     set ::etprof::exclusive($name) 0
     set ::etprof::cumulative($name) 0
     set ::etprof::depth($name) 0
 }

 proc ::etprof::init {} {
     rename ::proc ::etprof::oldProc
     rename ::etprof::profProc ::proc
     set ::etprof::timer [clock clicks]
     set ::etprof::hits 0
     array set ::etprof::exclusive {}
     array set ::etprof::cumulative {}
     array set ::etprof::calls {}
     set ::etprof::cumulative_timers {}
     ::etprof::initProcInfo TOPLEVEL
     set ::etprof::calls(TOPLEVEL) 1
     set ::etprof::cumulative(TOPLEVEL) {NOT AVAILABLE}
     return
 }

 proc ::etprof::printInfoLine {name exTot exTotPerc callsNum avgExPerCall cumulTot {sep |}} {
     puts [format "$sep%-17.17s$sep%14.14s$sep%6.6s$sep%8.8s$sep%14.14s$sep%14.14s$sep" \
        $name $exTot $exTotPerc% $callsNum $avgExPerCall $cumulTot]
 }

 proc ::etprof::printInfoSeparator {} {
     set hline [string repeat - 30]
     ::etprof::printInfoLine $hline $hline $hline $hline $hline $hline +
 }

 proc ::etprof::percentage {part total} {
     set p [expr {($part*100.0)/$total}]
     format "%.02f" $p
 }

 proc ::etprof::printLiveInfo {} {
     set info {}
     foreach {key val} [array get ::etprof::exclusive] {
        lappend info [list $key $val]
     }
     set info [lsort -decreasing -index 1 -integer $info]
     ::etprof::printInfoSeparator
     ::etprof::printInfoLine PROCNAME EXCLUSIVETOT {} CALLNUM AVGPERCALL CUMULTOT
     ::etprof::printInfoSeparator
     # Sum all the exclusive times to print infos in percentage
     set totalExclusiveTime 0
     foreach i $info {
        foreach {name exclusiveTime} $i break
        incr totalExclusiveTime $exclusiveTime
     }
     foreach i $info {
        foreach {name exclusiveTime} $i break
        set cumulativeTime $::etprof::cumulative($name)
        set calls $::etprof::calls($name)
        if {$calls} {
            set avgTimePerCall [expr {int($exclusiveTime/$calls)}]
        } else {
            set avgTimePerCall 0
        }
        if {$::etprof::depth($name)} {
            set cumulativeTime "(*)$cumulativeTime"
        }
        ::etprof::printInfoLine $name $exclusiveTime [::etprof::percentage $exclusiveTime $totalExclusiveTime] $calls $avgTimePerCall $cumulativeTime
     }
     ::etprof::printInfoSeparator
     puts "(*) = Incomplete result, program inside that function."
 }

 ::etprof::init

package require Tk
catch {console show}

proc reverse_list list {
  set res {}
  set i [llength $list]
  while {$i > 0} {lappend res [lindex $list [incr i -1]]}
  set res
} ;# RS

# VERSE 1 : Mid-hack first-cut at painting photo images.
if 0 {

proc add_to_shape_list {color x1 y1 x2 y2} {
  global shape_list
  lappend shape_list [list $color $x1 $y1 $x2 $y2]
}

proc draw_bg {} {
  global x
  $x put black -to 0 0 500 500
}

proc render_one {color x1 y1 x2 y2} {
  global x
  $x put $color -to $x1 $y1 $x2 $y2
}

proc render {} {
  global shape_list
  global x

  #puts "calling render [clock seconds] [clock clicks]"
  #draw_bg
  foreach el $shape_list {
    eval render_one $el
  }
}

proc add_shape {color x1 y1 x2 y2} {
  add_to_shape_list $color $x1 $y1 $x2 $y2
  render
}

proc who_got_clicked {x y} {
  global shape_list
  set hit ""
  set reversed_list [reverse_list $shape_list]
  foreach el $reversed_list {
    foreach {color x1 y1 x2 y2} $el break
    if {($x >= $x1) && ($x <= $x2) &&
        ($y >= $y1) && ($y <= $y2)} {
      set hit $color
      break
    }
  }
  return $hit
}

proc move_shape {active_color nx ny} {

  global shape_list

  # go thru the shape_list, find the matching color (which doubles as ID),
  # and add nx,ny to its coords.

  set index 0
  foreach el $shape_list {
    foreach {color x1 y1 x2 y2} $el break
    if {$color eq $active_color} {
      set vx1 [expr {$x1 + $nx}] ; set vy1 [expr {$y1 + $ny}]
      set vx2 [expr {$x2 + $nx}] ; set vy2 [expr {$y2 + $ny}]
      break
    }
    incr index
  }

if 0 {
  if {($vx1 < 0) || ($vy1 < 0) || ($vx2 < 0) || ($vy2 < 0)} return
}

  set shape_list [lreplace $shape_list $index $index]

  
  render_one black $x1 $y1 $x2 $y2
  add_shape $color $vx1 $vy1 $vx2 $vy2

}

proc clicked {X Y} {
  global active_shape
  global qx
  global qy

  set qx $X
  set qy $Y

  set x [expr $X - [winfo rootx .x]]
  set y [expr $Y - [winfo rooty .x]]
  set who [who_got_clicked $x $y]
  set active_shape $who
}

proc moving {X Y} {
  global active_shape
  global qx
  global qy

  if {$active_shape eq ""} return

  set nx [expr {$X - $qx}]
  set ny [expr {$Y - $qy}]
  set qx $X
  set qy $Y

  #puts "moved $active_shape by $nx $ny"

  move_shape $active_shape $nx $ny

}


proc init {} {
uplevel #0 {

set x [image create photo -width 500 -height 500]
pack [label .x -image $x -bd 0]

. conf -bg gray80
wm geom . 600x550


set shape_list {}

bind .x <1>          [list clicked %X %Y]
bind .x <B1-Motion>  [list moving %X %Y]

draw_bg


# the ID for the shape and the color is one and the same,
# which means that you can only have unique colored shapes.
#
add_shape green   100 100 200 200
add_shape purple  150 150 250 250

}
}


init

} ;# END VERSE

# VERSE 2 : Tiny example of full repainting of photo image.
if 0 {

set x [image create photo -width 500 -height 500]
pack [label .x -image $x -bd 0]

. conf -bg gray80
wm geom . 600x550


foreach {x1 y1 x2 y2} [list 200 200 300 300] break

proc render {} {
  global x
  global x1 y1 x2 y2
  $x put black  -to 0 0 500 500
  $x put orange -to $x1 $y1 $x2 $y2
}

proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  global x1 y1 x2 y2
  incr x1 $dx ; incr y1 $dy
  incr x2 $dx ; incr y2 $dy
  render
}


bind .x <1>               [list move_start   %X %Y]
bind .x <Button1-Motion>  [list move_motion  %X %Y]

render

} ;# END VERSE

#VERSE 3

if 0 {
set x [image create photo -width 500 -height 500]
pack [label .x -image $x -bd 0]

. conf -bg gray80
wm geom . 600x550


foreach {x1 y1 x2 y2} [list 200 200 300 300] break

proc move_orange_by {dx dy} {
  global x1 y1 x2 y2
  puts "original pos is $x1,$y1 to $x2,$y2"
  puts "want to move by $dx $dy"
}

proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  move_orange_by $dx $dy
}


bind .x <1>               [list move_start   %X %Y]
bind .x <Button1-Motion>  [list move_motion  %X %Y]

$x put black  -to 0 0 500 500
$x put orange -to $x1 $y1 $x2 $y2
}

# VERSE 4 : megaimage in full effect!  again full repainting of frame.
if 0 {

load [file join $dir megaimage.dll]
load [file join $dir megaimagetk.dll]

set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800

set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle $obj_x $obj_y $size $size {255 255 255 0}

pack [megaimage.frame .f] -fill both -expand 1
.f size $canvas_width $canvas_height
.f objsetimage $obj


proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  global obj
  global size
  global obj_x obj_y
  global canvas_width canvas_height

  # blank the canvas to black
  $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 0} 

  set obj_x [expr {$obj_x + $dx}]
  set obj_y [expr {$obj_y + $dy}]
  $obj rectangle $obj_x $obj_y $size $size {255 255 255 0}

  .f objsetimage $obj

}


bind .f <1>               [list move_start   %X %Y]
bind .f <Button1-Motion>  [list move_motion  %X %Y]

}

# NTK crash
if 0 {

load [file join $dir megaimage.dll]
load [file join $dir ntk.dll]

set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800


set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle $obj_x $obj_y $size $size {255 255 255 0}


set winid [ntk-create-sys-window 400 400 1]

ntk-map-sys-window $winid

ntk-put-image $winid $obj

}

# simple megaimage.put.on.photo working
if 0 {
load [file join $dir megaimage.dll]
load [file join $dir megaimagetk.dll]


set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800

set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
$obj rectangle $obj_x $obj_y $size $size {255 255 255 255}



set p [image create photo]


pack [label .p -width $canvas_width -height $canvas_height] \
  -fill both -expand 1
.p conf -image $p

megaimage.put.on.photo [$obj getdata] $p 0 0 set


}

# using megaimage.put.on.photo for animation, even slower.
#
if 0 {

load [file join $dir megaimage.dll]
load [file join $dir megaimagetk.dll]

set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800


set p [image create photo]
pack [label .p -width $canvas_width -height $canvas_height] \
  -fill both -expand 1
.p conf -image $p


set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255} 
$obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
megaimage.put.on.photo [$obj getdata] $p 0 0 set


proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  global obj
  global size
  global obj_x obj_y
  global canvas_width canvas_height
  global p


  set obj_x [expr {$obj_x + $dx}]
  set obj_y [expr {$obj_y + $dy}]

  $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255} 
  $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
  megaimage.put.on.photo [$obj getdata] $p 0 0 set


}


bind .p <1>               [list move_start   %X %Y]
bind .p <Button1-Motion>  [list move_motion  %X %Y]


}

# slowest with after idle and megaimage.put.on.photo
if 0 {

 set dir [file dirname [info script]]
 set ext [info sharedlibextension]
 load [file join $dir megaimage$ext]
 load [file join $dir megaimagetk$ext]
  
 set size 100
 set obj_x 100
 set obj_y 50
  
 set canvas_width 600
 set canvas_height 800
  
  
 set p [image create photo]
 pack [label .p -width $canvas_width -height $canvas_height] \
 -fill both -expand 1
 .p conf -image $p
  
  
 set obj [megaimage-blank $canvas_width $canvas_height]
 $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
 $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
 megaimage.put.on.photo [$obj getdata] $p 0 0 set
  
  
proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}
 

proc render {} {
 global render_pending obj size obj_x obj_y p

 $obj setall {0 0 0 255}
 $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
 megaimage.put.on.photo [$obj getdata] $p 0 0 set
 set render_pending ""
}

set ::render_pending ""
proc move_motion {X Y} {
 global qx qy
 set dx [expr {$X - $qx}]
 set dy [expr {$Y - $qy}]
 set qx $X ; set qy $Y
 global obj size obj_x obj_y canvas_width canvas_height p render_pending
  
 set obj_x [expr {$obj_x + $dx}]
 set obj_y [expr {$obj_y + $dy}]
  
 set render_pending [after idle render]
}
 
bind .p <1> [list move_start %X %Y]
bind .p <Button1-Motion> [list move_motion %X %Y] 

}

# using megaimage.frame but fixed alphas, same
if 0 {

load [file join $dir megaimage.dll]
load [file join $dir megaimagetk.dll]

set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800

set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
$obj rectangle $obj_x $obj_y $size $size {255 255 255 255}

pack [megaimage.frame .f] -fill both -expand 1
.f size $canvas_width $canvas_height
.f objsetimage $obj


proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  global obj
  global size
  global obj_x obj_y
  global canvas_width canvas_height

  set obj_x [expr {$obj_x + $dx}]
  set obj_y [expr {$obj_y + $dy}]

  $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
  $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
  .f objsetimage $obj

}


bind .f <1>               [list move_start   %X %Y]
bind .f <Button1-Motion>  [list move_motion  %X %Y]

}

# again using  megaimage.frame but with after idle  (GPS paste) 
# same speed as the other fastest
if 0 {

set dir [file dirname [info script]]
set ext [info sharedlibextension]
load [file join $dir megaimage$ext]
load [file join $dir megaimagetk$ext]

set size 100
set obj_x 100
set obj_y 50

set canvas_width  600
set canvas_height 800

set obj [megaimage-blank $canvas_width $canvas_height]
$obj rectangle $obj_x $obj_y $size $size {255 255 255 0}

pack [megaimage.frame .f] -fill both -expand 1
.f size $canvas_width $canvas_height
.f objsetimage $obj


proc move_start {X Y} {
  global qx qy
  set qx $X ; set qy $Y
}

set ::rendering ""

proc render {} {
  global obj size obj_x obj_y rendering
  # blank the canvas to black
  $obj setall {0 0 0 0} 
  $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
  .f objsetimage $obj

  set rendering ""
}

proc move_motion {X Y} {
  global qx qy
  set dx [expr {$X - $qx}]
  set dy [expr {$Y - $qy}]
  set qx $X ; set qy $Y

  global obj
  global size
  global obj_x obj_y
  global canvas_width canvas_height rendering
  set obj_x [expr {$obj_x + $dx}]
  set obj_y [expr {$obj_y + $dy}]
  
  if {"" eq $rendering} {
    set rendering [after idle render]
  }
}


bind .f <1>               [list move_start   %X %Y]
bind .f <Button1-Motion>  [list move_motion  %X %Y]

}

puts "drag mouse around and see the mouse move"

# paste from http://wiki.tcl.tk/20595

  set dir [file dirname [info script]]
  set ext [info sharedlibextension]
  load [file join $dir megaimage$ext]
  load [file join $dir megaimagetk$ext]

  set size 100
  set obj_x 100
  set obj_y 50

  set canvas_width  600
  set canvas_height 800

  set obj [megaimage-blank $canvas_width $canvas_height]
  $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
  $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}

  pack [megaimage.frame .f] -fill both -expand 1
  .f size $canvas_width $canvas_height
  .f objsetimage $obj

  proc move_start {X Y} {
    global qx qy
    set qx $X ; set qy $Y
  }

  proc move_motion {X Y} {
    global qx qy
    set dx [expr {$X - $qx}]
    set dy [expr {$Y - $qy}]
    set qx $X ; set qy $Y

    global obj
    global size
    global obj_x obj_y
    global canvas_width canvas_height

    set obj_x [expr {$obj_x + $dx}]
    set obj_y [expr {$obj_y + $dy}]

    $obj rectangle 0 0 $canvas_width $canvas_height {0 0 0 255}
    $obj rectangle $obj_x $obj_y $size $size {255 255 255 255}
    .f objsetimage $obj

  }

  bind .f <1>               [list move_start   %X %Y]
  bind .f <Button1-Motion>  [list move_motion  %X %Y]

